package com.sun.portal.pom;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Id;
import javax.persistence.IdClass;
import javax.persistence.OneToMany;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElements;

@javax.persistence.Entity
@IdClass(com.sun.portal.pom.PageId.class)
public class Page implements Serializable {
    @Id
    @Column (nullable=false, name="PAGE_NAME")
    @XmlAttribute (required = true)
    private String name;
    
    @OneToMany(cascade=CascadeType.ALL, mappedBy = "page")    
    @XmlElements ({
        @XmlElement (name="portlet-entity", type = PortletEntity.class),
        @XmlElement (name="local-entity", type = LocalEntity.class),
        @XmlElement (name="remote-entity", type = RemoteEntity.class),
        @XmlElement (name="decorator-entity", type = DecoratorEntity.class)
    })
    private List<Entity> entities = new ArrayList<Entity>();
        
    public String getName() {
        return name;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public List<Entity> getEntities() {
        return entities;
    }
    
    public void putEntity(Entity e) {
        e.setPage(this);
        entities.add(e);
    }
    
    public Entity getEntity(String name) {
        for (Entity e: entities) {
            if (e.getName().equals(name)) {
                return e;
            }
        }
        
        return null;
    }

    public DecoratorEntity getDecoratorEntity() {
        for (Entity e: getEntities()) {
            if (e instanceof DecoratorEntity) {
                return (DecoratorEntity)e;
            }
        }
        
        return null;
    }
    
    public PageId getPageId() {
        return new PageId(this);
    }
    
    public String toString() {
        return "{name=" + name + ", entities=" + entities.toString() + " }";
    }
}
