package com.sun.portal.pom;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import junit.framework.Test;
import junit.framework.TestSuite;

public class PersistTest extends PomTest {    
    public PersistTest( String testName ) {
        super( testName );
    }
    
    public static Test suite() {
        return new TestSuite( PersistTest.class );
    }
    
    protected void tearDown() throws java.lang.Exception {
        // nothing
    }
    
    private Entity findEntity(String pageId, String name) {
        EntityManager em = null;
        EntityTransaction et = null;
        EntityId id = new EntityId(pageId, name);
        
        try {
            EntityManagerFactory emf = Persistence.createEntityManagerFactory("pomtest");
            em = emf.createEntityManager();
            
            et = em.getTransaction();
            et.begin();
            
            Entity entity = em.find(Entity.class, id);
            //Entity entity = em.find(LocalEntity.class, id);
            
            return entity;
        } catch (Exception e) {
            if (et != null && et.isActive()) {
                et.rollback();
            }
            e.printStackTrace();
            fail(e.getMessage());
            
            return null;
        } finally {
            if (et != null && et.isActive()) {
                et.commit();
            }
            if (em != null && em.isOpen()) {
                em.close();
            }
        }
    }
    
    private Page findPage(String name) {
        EntityManager em = null;
        EntityTransaction et = null;
        PageId id = new PageId(name);
        
        try {
            EntityManagerFactory emf = Persistence.createEntityManagerFactory("pomtest");
            em = emf.createEntityManager();
            
            et = em.getTransaction();
            et.begin();
            
            Page page = em.find(Page.class, id);
            
            return page;
        } catch (Exception e) {
            if (et != null && et.isActive()) {
                et.rollback();
            }
            e.printStackTrace();
            fail(e.getMessage());
            
            return null;
        } finally {
            if (et != null && et.isActive()) {
                et.commit();
            }
            if (em != null && em.isOpen()) {
                em.close();
            }
        }
    }
    
    private void persistEntity(Entity entity) {
        EntityManager em = null;
        EntityTransaction et = null;
        
        try {
            EntityManagerFactory emf = Persistence.createEntityManagerFactory("pomtest");
            em = emf.createEntityManager();
            
            et = em.getTransaction();
            et.begin();
            
            em.persist(entity);
        } catch (Exception e) {
            if (em != null && em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            e.printStackTrace();
            fail(e.getMessage());
        } finally {
            if (et != null && et.isActive()) {
                et.commit();
            }
            if (em != null && em.isOpen()) {
                em.close();
            }
        }
    }

    private void persistPage(Page page) {
        EntityManager em = null;
        EntityTransaction et = null;
        
        try {
            EntityManagerFactory emf = Persistence.createEntityManagerFactory("pomtest");
            em = emf.createEntityManager();
            
            et = em.getTransaction();
            et.begin();
            
            em.persist(page);
        } catch (Exception e) {
            if (em != null && em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            e.printStackTrace();
            fail(e.getMessage());
        } finally {
            if (et != null && et.isActive()) {
                et.commit();
            }
            if (em != null && em.isOpen()) {
                em.close();
            }
        }
    }
    
    private void removeEntity(String id, boolean assertRemove) {
        EntityManager em = null;
        EntityTransaction et = null;
        
        try {
            EntityManagerFactory emf = Persistence.createEntityManagerFactory("pomtest");
            em = emf.createEntityManager();
            
            et = em.getTransaction();
            et.begin();
            //entity = em.find(Entity.class, entity.getId());
            Entity entity = em.find(LocalEntity.class, id);
            if (entity != null) {
                em.remove(entity);            
            } else if (assertRemove) {
                fail("entity did not exist: " + entity);
            }
        } catch (Exception e) {
            if (em != null && em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            e.printStackTrace();
            fail(e.getMessage());
        } finally {
            if (et != null && et.isActive()) {
                et.commit();
            }
            if (em != null && em.isOpen()) {
                em.close();
            }
        }
    }

    private void removePage(String name, boolean assertRemove) {
        EntityManager em = null;
        EntityTransaction et = null;
        PageId id = new PageId(name);
        
        try {
            EntityManagerFactory emf = Persistence.createEntityManagerFactory("pomtest");
            em = emf.createEntityManager();
            
            et = em.getTransaction();
            et.begin();
            Page page = em.find(Page.class, id);
            if (page != null) {
                em.remove(page);            
            } else if (assertRemove) {
                fail("page did not exist: " + page);
            }
        } catch (Exception e) {
            if (em != null && em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            e.printStackTrace();
            fail(e.getMessage());
        } finally {
            if (et != null && et.isActive()) {
                et.commit();
            }
            if (em != null && em.isOpen()) {
                em.close();
            }
        }
    }
            
    public void testPersist() throws Exception {  
        // clean up from a previous failed test
        // ???
        
        Pom pom = unmarshalPom();
        System.out.println(pom);
        
        for (Page p: pom.getPages()) {
            persistPage(p);
        }
        
        Page p = findPage("page1");
        List<Entity> entities = p.getEntities();
        assertNotNull(entities);
        assertEquals(4, entities.size());
        
        PortletEntity pe = (PortletEntity)findEntity(p.getName(), "AddressBook");
        assertNotNull(pe);
        List<Preference> prefs = pe.getPreferences();
        assertNotNull(prefs);
        assertEquals(6, prefs.size());
        
        LocalEntity le = (LocalEntity)findEntity(p.getName(), "RSS");
        assertNotNull(le);
        RemoteEntity re = (RemoteEntity)findEntity(p.getName(), "HR");
        assertNotNull(re);
        DecoratorEntity de = (DecoratorEntity)findEntity(p.getName(), "SimpleDecorator");
        assertNotNull(de);
        
        // clean up
        removePage("page1", true);
    }
}
